@@ -5,7 +5,7 @@ from typing import Any
 import pandas as pd
 import pytz
 from pydantic import BaseModel, ValidationError
-from pydantic.fields import FieldInfo
+from pydantic.fields import FieldInfo, Undefined
 
 from rframe.utils import singledispatchmethod
 
@@ -15,7 +15,7 @@ class BaseIndex(FieldInfo):
     
     __slots__ = FieldInfo.__slots__ + ("name", "schema", "field", "nullable", "unique")
 
-    def __init__(self, default: Any = ..., **kwargs: Any) -> None:
+    def __init__(self, default: Any = Undefined, **kwargs: Any) -> None:
         self.nullable = kwargs.pop("nullable", True)
         self.unique = kwargs.pop("unique", True)
         super().__init__(default, **kwargs)
@@ -30,9 +30,14 @@ class BaseIndex(FieldInfo):
 
     @property
     def names(self):
+        '''returns a list of names
+        for compatibility with multi-index
+        '''
         return [self.name]
 
     def _validate_label(self, label):
+        '''Perform label validation
+        '''
         if label is None:
             return label
             
@@ -69,31 +74,6 @@ class BaseIndex(FieldInfo):
 
         return self._validate_label(label)
 
-    def coerce(self, label):
-        if isinstance(label, self.label_type):
-            return label
-
-        label = self._coerce(self.label_type, label)
-
-        return label
-
-    @singledispatchmethod
-    def _coerce(self, type_, value):
-        return type_(value)
-
-    @_coerce.register(datetime)
-    def _coerce_datetime(self, type_, value):
-        if isinstance(value, datetime):
-            if value.tzinfo is not None and value.tzinfo.utcoffset(value) is not None:
-                value = value.astimezone(pytz.utc)
-            else:
-                value = value.replace(tzinfo=pytz.utc)
-            return value
-        unit = self.unit if isinstance(value, numbers.Number) else None
-        utc = getattr(self, "utc", True)
-        value = pd.to_datetime(value, utc=utc, unit=unit).to_pydatetime()
-        return self._coerce_datetime(type_, value)
-
     def to_pandas(self, label):
         return label
 
