@@ -5,6 +5,8 @@ from .base import DatasourceInterface
 from .mongo import MongoInterface
 from .pandas import PandasInterface
 from .rest import RestInterface
+from .tinydb import TinyDBInterface
+from .json import JsonInterface
 
 
 def get_interface(source, **kwargs) -> DatasourceInterface:
